<?php

/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

namespace Sources\Libraries\Version;

/**
 * A class that defines a single 4-integer version with a single version type
 *
 * @author Cruz Julian Bishop
 * @since 0.0.0.1
 */
class Version {
    
    /**
     * Constructs a new version
     * 
     * @param int $major The major number to use
     * @param int $minor The minor number to use
     * @param int $build The build number to use
     * @param int $revision The revision number to use
     * @param \Sources\Libraries\Version\VersionType $type The version type to use
     * @param string $codeName The code name to use (Optional)
     * @since 0.0.0.1
     */
    public function __construct($major, $minor, $build, $revision, $type, $codeName = "") {
        $this->setMajor($major);
        $this->setMinor($minor);
        $this->setBuild($build);
        $this->setRevision($revision);
        $this->setType($type);
        $this->setCodeName($codeName);
    }
    
    /**
     * The major version number
     * 
     * @var int The major version number
     * @since 0.0.0.1
     */
    private $major;
    
    /**
     * Gets the major version number
     * 
     * @return int The major version number
     * @since 0.0.0.1
     */
    public function getMajor() {
        return $this->major;
    }
    
    /**
     * Sets the major version number
     * 
     * @param int $major The version number to use
     * @since 0.0.0.1
     */
    public function setMajor($major) {
        //Do a quick sanity check
        if ($major < 0)
            $major = 0;
        $this->major = $major;
    }
    
    /**
     * The minor version number
     * 
     * @var int The minor version number
     * @since 0.0.0.1
     */
    private $minor;
    
    /**
     * Gets the minor version number
     * 
     * @return int The minor version number
     * @since 0.0.0.1
     */
    public function getMinor() {
        return $this->minor;
    }
    
    /**
     * Sets the minor version number
     * 
     * @param int $minor The version number to use
     * @since 0.0.0.1
     */
    public function setMinor($minor) {
        //Do a quick sanity check
        if ($minor < 0) {
            $minor = 0;
        }
        $this->minor = $minor;
    }
    
    /**
     * The build number
     * 
     * @var int The build number
     * @since 0.0.0.1
     */
    private $build;
    
    /**
     * Gets the build number
     * 
     * @return int The build number
     * @since 0.0.0.1
     */
    public function getBuild() {
        return $this->build;
    }
    
    /**
     * Sets the build number
     * 
     * @param int $build The build number to use
     * @since 0.0.0.1
     */
    public function setBuild($build) {
        //Do a quick sanity check
        if ($build < 0) {
            $build = 0;
        }
        $this->build = $build;
    }
    
    /**
     * The revision number
     * 
     * @var int The revision number
     * @since 0.0.0.1
     */
    private $revision;
    
    /**
     * Gets the revision number
     * 
     * @return int The revision number
     * @since 0.0.0.1
     */
    public function getRevision() {
        return $this->revision;
    }
    
    /**
     * Sets the revision number
     * 
     * @param int $revision The revision number to use
     * @since 0.0.0.1
     */
    public function setRevision($revision) {
        //Do a quick sanity check
        if ($revision < 0) {
            $revision = 0;
        }
        $this->revision = $revision;
    }
    
    /**
     * The version type
     * 
     * @var \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    private $type;
    
    /**
     * Gets the type
     * 
     * @return \Sources\Libraries\Version\VersionType The type
     * @since 0.0.0.1
     */
    public function getType() {
        return $this->type;
    }
    
    /**
     * Sets the type
     * 
     * @param \Sources\Libraries\Version\VersionType $type The type to use
     * @since 0.0.0.1
     */
    public function setType($type) {
        $this->type = $type;
    }
    
    /**
     * The code name of this version
     * 
     * @var string The code name
     * @since 0.0.0.2
     */
    private $codeName;
    
    /**
     * Gets the code name of this version
     * 
     * @return string The code name
     * @since 0.0.0.2
     */
    public function getCodeName() {
        return $this->codeName;
    }
    
    /**
     * Sets the code name of this version
     * 
     * @param string $codeName The code name ot use
     * @since 0.0.0.2
     */
    public function setCodeName($codeName) {
        $this->codeName = $codeName;
    }
    
    /**
     * Gets this version as a string
     * 
     * @return string The string
     * @since 0.0.0.1
     */
    public function __toString() {
        return $this->major . "." .
                $this->minor . "." .
                $this->build . "." .
                $this->revision .
                $this->type->getPostfix() . (empty($this->codeName) ? "" : " \"$this->codeName\"");
    }
    
}

?>
