<?php

/*
 * The MIT License
 *
 * Copyright 2012 Massive Dynamics.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

namespace Sources\Libraries\Version;

/**
 * A class that defines multiple version types
 *
 * @author Cruz Julian Bishop
 * @since 0.0.0.1
 */
class VersionType {
    
    /**
     * Creates a new version type
     *
     * @param int $order The order in the list to use
     * @param string $name The name to use
     * @param string $postfix The postfix to use
     * @since 0.0.0.1
     */
    public function __construct($order, $name, $postfix = "") {
        $this->setOrder($order);
        $this->setName($name);
        $this->setPostfix($postfix);
    }
    
    //<editor-fold desc="Dynamic functions">
    
    /**
     * The order
     * 
     * @var int The order to use
     * @since 0.0.0.1
     */
    private $order;
    
    /**
     * Gets the order to use
     * 
     * @return int The order to use
     * @since 0.0.0.1 
     */
    public function getOrder() {
        return $this->order;
    }
    
    /**
     * Sets the order
     * 
     * @param int $order The order to use
     * @since 0.0.0.1
     */
    public function setOrder($order) {
        $this->order = $order;
    }
    
    /**
     * The name
     * 
     * @var string The name to use
     * @since 0.0.0.1
     */
    private $name;
    
    /**
     * Gets the name to use
     * 
     * @return string The name to use
     * @since 0.0.0.1 
     */
    public function getName() {
        return $this->name;
    }
    
    /**
     * Sets the name
     * 
     * @param string $name The name to use
     * @since 0.0.0.1
     */
    public function setName($name) {
        $this->name = $name;
    }
    
    /**
     * The postfix
     * 
     * @var string The postfix to use
     * @since 0.0.0.1
     */
    private $postfix;
    
    /**
     * Gets the postfix to use
     * 
     * @return string The postfix to use
     * @since 0.0.0.1 
     */
    public function getPostfix() {
        return $this->postfix;
    }
    
    /**
     * Sets the postfix
     * 
     * @param string $postfix The postfix to use
     * @since 0.0.0.1
     */
    public function setPostfix($postfix) {
        $this->postfix = $postfix;
    }
    
    //</editor-fold>
    
    //<editor-fold desc="Static functions">
    
    /**
     * Gets a version type by postfix
     * 
     * @param string $postfix The postfix
     * @return \Sources\Libraries\Version\VersionType The version type
     */
    public static function getType($postfix = "") {
        //Start switching by postfix
        switch($postfix) {
            
            //Try RC
            case "-RC":
            case "-ReleaseCandidate":
                return static::getReleaseCandidateType();
            break;
        
            //Try BETA
            case "-BETA":
                return static::getBetaType();
            break;
        
            //Try alpha
            case "-ALPHA":
                return static::getAlphaType();
            break;
        
            //Finally, try snapshot
            case "-SNAPSHOT":
            case "-DAILY":
                return static::getSnapshotType();
            break;
        
            //Nothing found, get a default value of STABLE
            default:
                return static::getStableType();
            break;
        }
    }
    
    /**
     * The snapshot type
     * 
     * @var \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    private static $snapshotType = null;
    
    /**
     * Gets the snapshot type
     * 
     * @return \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    public static function getSnapshotType() {
        //Check to see if the snapshot type is null
        if (is_null(static::$snapshotType)) {
            //Set it
            static::$snapshotType = new VersionType(0, "Snapshot", "-SNAPSHOT");
        }
        
        //Return the type
        return static::$snapshotType;
    }
    
    /**
     * The alpha type
     * 
     * @var \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    private static $alphaType = null;
    
    /**
     * Gets the alpha type
     * 
     * @return \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    public static function getAlphaType() {
        //Check to see if the alpha type is null
        if (is_null(static::$alphaType)) {
            //Set it
            static::$alphaType = new VersionType(5, "Alpha", "-ALPHA");
        }
        
        //Return the type
        return static::$alphaType;
    }
    
    /**
     * The beta type
     * 
     * @var \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    private static $betaType = null;
    
    /**
     * Gets the beta type
     * 
     * @return \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    public static function getBetaType() {
        //Check to see if the beta type is null
        if (is_null(static::$betaType)) {
            //Set it
            static::$betaType = new VersionType(0, "Beta", "-BETA");
        }
        
        //Return the type
        return static::$betaType;
    }
    
    /**
     * The Release Candidate type
     * 
     * @var \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    private static $releaseCandidateType = null;
    
    /**
     * Gets the Release Candidate type
     * 
     * @return \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    public static function getReleaseCandidateType() {
        //Check to see if the Release Candidate type is null
        if (is_null(static::$releaseCandidateType)) {
            //Set it
            static::$releaseCandidateType = new VersionType(0, "Release Candidate", "-RC");
        }
        
        //Return the type
        return static::$releaseCandidateType;
    }
    
    /**
     * The stable type
     * 
     * @var \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    private static $stableType = null;
    
    /**
     * Gets the stable type
     * 
     * @return \Sources\Libraries\Version\VersionType The version type
     * @since 0.0.0.1
     */
    public static function getStableType() {
        //Check to see if the stable type is null
        if (is_null(static::$stableType)) {
            //Set it
            static::$stableType = new VersionType(0, "Stable");
        }
        
        //Return the type
        return static::$stableType;
    }
    
    //</editor-fold>
    
}

?>
